#version 410 compatibility

#include "/lib/distort.glsl"
#include "/lib/common.glsl"


uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D shadowtex0;
uniform sampler2D shadowtex1;
uniform sampler2D shadowcolor0;


uniform float far;
vec3 dayDistFogColor = vec3(0.2078, 0.349, 0.7333);
vec3 earlyDistFogColor = vec3(0.702, 0.3804, 0.1765);
vec3 duskDistFogColor = vec3(0.8196, 0.2549, 0.098);
vec3 nightDistFogColor = vec3(0.051, 0.051, 0.1451);
vec3 rainFogColor = vec3(1.0, 1.0, 1.0);
vec3 vlFogColor = vec3(0.7882, 0.7216, 0.4745);
uniform int worldTime;
uniform mat4 gbufferProjectionInverse;
uniform int isEyeInWater;
uniform int frameCounter;

in vec2 texcoord;
uniform float wetness;
const float wetnessHalflife = 600.0;

vec3 projectAndDivide(mat4 projectionMatrix, vec3 position){
  vec4 homPos = projectionMatrix * vec4(position, 1.0);
  return homPos.xyz / homPos.w;
}

/* RENDERTARGETS: 0 */
layout(location = 0) out vec4 color;

void main() {
  color = texture(colortex0, texcoord);

  float depth = texture(depthtex1, texcoord).r;
  if(depth == 1.0){
    return;
  }

  #if DO_DISTANCE_FOG == 1
  float farPlane = far * 4;
  
  vec3 NDCPos = vec3(texcoord.xy, depth) * 2.0 - 1.0;
  vec3 viewPos = projectAndDivide(gbufferProjectionInverse, NDCPos);

  // Fog calculations
  float dist = length(viewPos) / far;
  float fogFactor = exp(-FOG_DENSITY * (1.0 - dist));
  float nightFogFactor = exp(-FOG_DENSITY * (0.073 / dist));
  
  vec3 fogColor = dayDistFogColor;

	//Time of day color changes
  if(worldTime >= 0 && worldTime < 1000)
	{
	 	float time = smoothstep(0, 1000, float(worldTime));
	 	fogColor = mix(earlyDistFogColor,dayDistFogColor, time);
	}
    else if(worldTime >= 10000 && worldTime < 11500)
     {
        float time = smoothstep(10000, 11500, float(worldTime));
        fogColor = mix(dayDistFogColor, duskDistFogColor, time);
       
    }
    else if(worldTime >= 11500 && worldTime < 13000)
     {
        float time = smoothstep(11900, 13000, float(worldTime));
        fogColor = mix(duskDistFogColor, nightDistFogColor/2, time);
        fogFactor = mix(fogFactor, nightFogFactor, time );
    }
    else if (worldTime >= 13000 && worldTime < 24000)
  {
      float time = smoothstep(23215, 24000, float(worldTime));
      fogColor = mix(nightDistFogColor/2, earlyDistFogColor, time);
      fogFactor = mix(nightFogFactor, fogFactor, time );
  }
  if(isEyeInWater == 1)
  {
    fogFactor = exp(-FOG_DENSITY * (7.0 - dist));
    color.rgb = mix(color.rgb, fogColor, clamp(fogFactor, 0.0, 7.0));
  }

  color.rgb = mix(color.rgb, fogColor, clamp(fogFactor, 0.0, 7.0));
#endif
}